Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
func ProveMultipleKeyShareEncryption ¶
func ProveMultipleKeyShareEncryption(css constraint.ConstraintSystem, provingKey *groth16.ProvingKey, sender [20]byte, pubKey []*ecies.PublicKey, rs []*big.Int, fisInts []*big.Int, encryptedFis [][]byte, nonces [][]byte) (*groth16.Proof, witness.Witness, error)
*
- Function: ProveMultipleKeyShareEncryption
- @Description: generate a zk proof of a key share batch generating process
- @param css: compiled circuit constraint system
- @param provingKey: proving key used for proof encryption
- @param sender: the sender address as identifier
- @param pubKey: a set of public keys used for key share encryption
- @param rs: a set of the integer format of random numbers
- @param fisBytes: a set of the serialization format of the keys
- @param fisInts: a set of the integer format of the keys
- @param encryptedFis: a set of encrypted key shares
- @param nonces: a set of salt
- @return proof: zk proof
- @return witness: witness of zk proof
- @return err:
func ProveSingleKeyShareEncryption ¶
func ProveSingleKeyShareEncryption(css constraint.ConstraintSystem, provingKey *groth16.ProvingKey, sender [20]byte, pubKey *ecies.PublicKey, r *big.Int, fiInt *big.Int, encryptedFi []byte, nonce []byte) (*groth16.Proof, witness.Witness, error)
*
- Function: ProveSingleKeyShareEncryption
- @Description: generate a zk proof of a key share generating process
- @param css: compiled circuit constraint system
- @param provingKey: proving key used for proof encryption
- @param sender: the sender address as identifier
- @param pubKey: public key used for key share encryption
- @param r: the integer format of random number
- @param fiBytes: the key share in a byte array
- @param fiInt: the integer format of the key share
- @param encryptedFi: the encrypted key share
- @param nonce: salt
- @return proof: zk proof
- @return witness: witness of zk proof
- @return err:
Types ¶
This section is empty.
Click to show internal directories.
Click to hide internal directories.